# Copyright 2018 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Refer to the README and COPYING files for full details of the license
import re

from ovirtsdk4.types import Bonding, HostNic, Option

SYSTEM_NAME_TYPE = 5
PORT_VLAN_TYPE = 127
PORT_VLAN_OUI = 0x0080c2
PORT_VLAN_SUBTYPE = 3
PROPERTY_VLAN_NAME = "VLAN ID"

LABEL_PREFIX = "lldp_vlan_"
BOND_PREFIX = "bond"
BOND_MODE_OPTION_NAME = "mode"
BOND_MODE_OPTION_VALUE = "1"


def filter_vlan_tag(tlvs):
    filtered_tlvs = []
    for tlv in tlvs:
        if tlv.type == PORT_VLAN_TYPE and tlv.oui == PORT_VLAN_OUI and tlv.subtype == PORT_VLAN_SUBTYPE:
            filtered_tlvs.extend(tlv.properties)
    return filtered_tlvs


def create_label_candidates(tlv_properties):
    label_candidates = []
    for property in tlv_properties:
        if property.name == PROPERTY_VLAN_NAME:
            label_candidates.append(LABEL_PREFIX + property.value)
    return label_candidates


def get_or_query(param, values):
    return ' OR '.join(['{}={}'.format(param, val) for val in values])


def filter_out_vlan_interfaces(nic_list):
    return [nic for nic in nic_list if nic.vlan is None]


def filter_out_bond_slaves(nic_list):
    slave_list = []
    for nic in nic_list:
        if nic.bonding is not None:
            slave_list.extend([slave.id for slave in nic.bonding.slaves])
    return [nic for nic in nic_list if nic.id not in slave_list]


def filter_system_name(tlvs):
    for tlv in tlvs:
        if tlv.type == SYSTEM_NAME_TYPE:
            return tlv.properties


def find_next_bond_num(nic_names):
    bond_names = [nic_name for nic_name in nic_names if re.search(r'bond.*', nic_name)]
    if len(bond_names) == 0:
        return 0
    else:
        bond_names.sort()
        return int(bond_names[0].strip('bond')) + 1


def update_bond_dict(bond_dict, tlvs, nic_id):
    system_name_tlv = filter_system_name(tlvs)
    if system_name_tlv is not None and len(system_name_tlv):
        system_name = system_name_tlv[0].value
        slave_list = bond_dict.get(system_name, None)
        if slave_list is None:
            bond_dict.update({system_name: [nic_id]})
        else:
            slave_list.append(nic_id)


def create_bond_definition(nic_id_list, next_bond_num):
    slaves_list = [HostNic(id=nic_id) for nic_id in nic_id_list]
    if len(slaves_list) > 1:
        bonding = Bonding(options=[Option(name=BOND_MODE_OPTION_NAME, value=BOND_MODE_OPTION_VALUE)],
                          slaves=slaves_list)
        return HostNic(name=BOND_PREFIX + str(next_bond_num), bonding=bonding)
    else:
        return None
